# Documentation string. I wonder what ever refers to it? 

__doc__ = """KZMailman product module.

See README.txt for more details."""

__version__ = '0.1.1'  

from Globals import HTMLFile      # fakes a method from a DTML file
from Globals import MessageDialog # provid
from Globals import Persistent    # makes an object stick in the ZODB
from Globals import InitializeClass

import OFS.SimpleItem
import Acquisition
import AccessControl.Role

import os, sys, httplib, urllib, urlparse
import popen2
import string
import tempfile

from AccessControl import ClassSecurityInfo

class KMMException(Exception) : pass

manage_addKZMailmanForm = HTMLFile('KZMailmanAdd', globals()) 

def manage_addKZMailman(self, id, title='', addAliases=None, aliasesPath='', mailmanPath='', mailmanURL='', sitePassword='', REQUEST=None):
    """Add a KZMailman instance to a folder."""
    if addAliases != None :
       addAliases = 1
    else :
       addAliases = 0
    self._setObject(id, KZMailman(id, title, addAliases, aliasesPath, mailmanPath, mailmanURL, sitePassword))
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)

class KZMailman(
    OFS.SimpleItem.Item,   # A simple Principia object. Not Folderish. 
    Persistent,            # Make us persistent. Yaah! 
    Acquisition.Implicit,  # Uh, whatever. 
    AccessControl.Role.RoleManager # Security manager. 
    ): 
    """KZMailman object.  This object performs the integration between
       Zope and Mailman. 
    """
        
    meta_type = 'KZMailman' 
    security = ClassSecurityInfo()

    manage_options = ( # what management options are there?
	{'label': 'Edit',       'action': 'manage_main'},
	{'label': 'View',       'action': ''}, # defaults to index_html
        {'label': 'Manage List Members', 'action':'manage_list_members'},
        {'label': 'Manage List Options', 'action':'manage_list_options'},
	{'label': 'Security',   'action': 'manage_access'},
        ) 

    __ac_permissions__=( # what permissions make sense for us? 
	('View management screens', ('manage_tabs','manage_main')),
	('Change permissions',      ('manage_access',)           ),
	('Admin KZMailman'     ,     ('manage_edit', 'createMailList', 'deleteMailList', 'getLists') ),
	('View KZMailman',            ('',)                        ),
        ('Add Mailman List Member', ('addMember',)           ),
        ('Remove Mailman List Member', ('removeMember',)        ),
        ('Get Mailman List Members', ('getMembers') ),
        ('Change Mailman List Options', ('changeListOptions',)       )
	)

    def __init__(self, id, title='', addAliases=0, aliasesPath='', mailmanPath='', mailmanURL='', sitePassword=''): 
        """initialise a new instance of KZMailman"""
        self.id = id
        self.title = title
        self.addAliases = addAliases
        self.aliasesPath = aliasesPath
        self.mailmanPath = mailmanPath
        self.mailmanURL = mailmanURL
        urlParts = urlparse.urlparse(self.mailmanURL)
        self._mailmanURLServer = urlParts[1]
        self._mailmanURLRoot = urlParts[2]
        self.sitePassword = sitePassword

    index_html = HTMLFile('index', globals()) # View Interface
    
    manage_main = HTMLFile('KZMailmanEdit', globals()) # Management Interface
    
    def manage_edit(self, title, REQUEST=None): 
        """does this really need a doc string?"""
        self.title = title
	if REQUEST is not None:
            if REQUEST.get('addAliases') != None :
               self.addAliases = 1
            else :
               self.addAliases = 0
            self.mailmanPath = REQUEST.get('mailmanPath')
            if self.mailmanPath[-1] == '/' :           # remove trailing '/'
               self.mailmanPath = self.mailmanPath[0:-1]
            self.mailmanURL = REQUEST.get('mailmanURL')
            if self.mailmanURL[-1] == '/' :            # remove trailing '/'
               self.mailmanURL = self.mailmanURL[0:-1]
            urlParts = urlparse.urlparse(self.mailmanURL)
            self._mailmanURLServer = urlParts[1]
            self._mailmanURLRoot = urlParts[2]
            self.aliasesPath = REQUEST.get('aliasesPath')
            self.sitePassword = REQUEST.get('sitePassword')
            return self.manage_main(self, REQUEST, manage_tabs_message="Properties updated.")  

    manage_list_members = HTMLFile('KZMailmanListMembers', globals()) # Management Interface
    manage_list_options = HTMLFile('KZMailmanListOptions', globals()) # Management Interface


    # optionItems is a dictionary that describes each of the available list options in
    # the following format:
    #
    #  key : (optionName, optionDescription, optionType, optionChoices)
    # 
    #    where...
    #  
    #  key is the name of the option in the config file
    #  optionName is text
    #  optionDescription is text
    #  optionType is (int, string, list, or enum)
    #  optionChoices is a tuple of tuples  if the optionType is enum ((choiceValue1, choiceText1),...)
    optionItems = {'real_name' : ('List Name', 'Friendly name of the list', 'string', ()),
                   'owner'     : ('List Owner', 'List admins email address(es)', 'list', ()),
                   'description' : ('List Description', 'Short Description', 'string', ()),
                   'info' : ('List Info', 'Long Description', 'string', ()),
                   'subject_prefix' : ('List Prefix', 'Text prepended to subject line', 'string', ()),
                   'welcome_msg' : ('Welcome Message', 'Added to front of welcome message', 'string', ()),
                   'goodbye_msg' : ('Goodbye Message', 'Added to front of goodbye message', 'string', ()),
                   'reply_goes_to_list' : ('Reply Goes To', 'To whom the reply goes', 'enum', ((0, 'Poster'), (1, 'This list'), (2, 'Explicit Address'))),
                   'reply_to_address' : ('Reply Address', 'Reply address', 'string', ()),
                   'administrivia' : ('Administrivia', 'Check for Administrative requests', 'enum', ((0, 'No'),(1, 'Yes'))),
                   'umbrella_list' : ('Umbrella List', 'This list is an umbrella list', 'enum', ((0, 'No'),(1, 'Yes'))),
                   'umbrella_member_suffix' : ('Umbrella Suffix', 'Umbrella suffix', 'string', ()),
                   'send_reminders' : ('Send Monthly Reminders', 'Send monthly reminders', 'enum', ((0, 'No'),(1, 'Yes'))),
                   'send_welcome_msg' : ('Send Welcome Message', 'Send message to new member', 'enum', ((0, 'No'),(1, 'Yes'))),
                   'admin_immed_notify' : ('Notify of Admin Tasks Immediatlely', 'Notify of Admin Tasks Immediately', 'enum', ((0, 'No'),(1, 'Yes'))),
                   'admin_notify_mchanges' : ('Notify Admin of Subscribe / Unsubscribe', '', 'enum', ((0, 'No'),(1, 'Yes'))),
                   'dont_respond_to_post_requests' : ('Dont respond to mail limit triggers', '', 'enum', ((0, 'Yes'), (1, 'No'))),
                   'max_message_size' : ('Max message size (Kb)', '', 'int', ()),
                   'host_name' : ('Host name', '', 'string', ()),
                   'web_page_url' : ('Mailman Web Page URL', '', 'string', ()),
                   'advertised' : ('Advertise this list', 'List on mailman home page', 'enum', ((0, 'No'),(1, 'Yes'))),
                   'subscribe_policy' : ('Subscription Policy', '', 'enum', ((1, 'Confirm'), (2, 'Require Approval'), (3, 'Confirm + Approval'))),
                   'private_roster' : ('Roster Availability', 'Who can view the roster', 'enum', ((0, 'Anyone'), (1, 'List members'), (2, 'List Administor Only'))),
                   'obscure_addresses' : ('Obscure email addresses', '', 'enum', ((0, 'No'),(1, 'Yes'))),
                   'moderated' : ('Moderated list', 'Must posts be approved by administrator?', 'enum', ((0, 'No'),(1, 'Yes'))),
                   'member_posting_only' : ('Member posting only', '', 'enum', ((0, 'No'),(1, 'Yes'))),
                   'posters' : ('Who can post?' , '', 'list', ()),
                   'require_explicit_destination' : ('Require explicity destination', '', 'enum', ((0, 'No'),(1, 'Yes'))),
                   'acceptable_aliases' : ('Alternate acceptable addresses', '', 'string', ()),
                   'max_num_recipients' : ('Max number of recipients' , '', 'int', ()),
                   'forbidden_posters' : ('Forbidden Posters', '', 'list', ()),
                   'anonymous_list' : ('Anonymous List', '', 'enum' , ((0, 'No'),(1, 'Yes'))),
                   'msg_header' : ('Message Header', '', 'string', ()),
                   'digestable' : ('Can members receive digests?', '', 'enum', ((0, 'No'),(1, 'Yes'))),
                   'digest_is_default' : ('Default delivery mode', '', 'enum', ((0, 'Regular'), (1, 'Digest'))),
                   'mime_is_default_digest' : ('Default digest mode', '', 'enum', ((0, 'Plain'), (1, 'Mime'))),
                   'digest_size_threshhold' : ('Size of digest in Kb before being sent', '', 'int', ()),
                   'digest_send_periodic' : ('Send digest before size threshhold?', '', 'enum', ((0, 'No'),(1, 'Yes'))),
                   'digest_header' : ('Digest Header', '', 'string', ()),
                   'bounce_processing' : ('Bounce Processing', '', 'enum', ((0, 'No'),(1, 'Yes'))), 
                   'minimum_removal_date' : ('Minimum days before bounce action', '', 'int', ()),
                   'max_posts_between_bounces' : ('Max number of messages in an hour (for bounce)', '', 'int', ()),
                   'automatic_bounce_action' : ('Automatic Bounce Action', '', 'enum', ((0, 'Do nothing'),(1, 'Disable and notify me'),(2, 'Disable and do not notify me'),(3, 'Remove and notify me'))),
                   'archive' : ('Archive messages', '', 'enum', ((0, 'No'),(1, 'Yes'))),
                   'archive_private' : ('Archive Type', '', 'enum', ((0, 'Public'), (1, 'Private'))),
                   'clobber_date' : ('Date in archive', '', 'enum', ((0, 'When sent'), (1, 'When resent'))),
                   'archive_volume_frequency' : ('How often to start a new archive', '', 'enum', ((0, 'Yearly'),(1, 'Monthly'),(2, 'Quarterly'),(3, 'Weekly'),(4, 'Daily'))),
                   'nntp_host' : ('NNTP Host', '', 'string', ()),
                   'linked_newsgroup' : ('Usenet Newsgroup', '', 'string', ()),
                   'gateway_to_news' : ('Send new posts to newsgroup?', '', 'enum', ((0, 'No'),(1, 'Yes'))),
                   'gateway_to_mail' : ('Send newsgroup posts to maillist?', '', 'enum', ((0, 'No'),(1, 'Yes'))),
                   'autorespond_postings' : ('Send Auto-response', '', 'enum', ((0, 'No'),(1, 'Yes'))),
                   'autoresponse_postings_text' : ('Auto response text', '', 'string', ()),
                   'autorespond_admin' : ('Auto respond to admin postings?', '', 'enum', ((0, 'No'),(1, 'Yes'))),
                   'autoresponse_admin_text' : ('Auto response admin text', '', 'string', ()),
                   'autorespond_requests' : ('Auto repond to request postings', '', 'enum', ((0, 'No'),(1, 'Yes w/discard'), (2, 'Yes w/forward'))),
                   'autoresponse_request_text' : ('Auto reponse request text', '', 'string', ()),
                   'autoresponse_graceperiod' : ('Number of days until auto response', '', 'int', ())
                  }

#############################################################################################
# 
# createMailList : This function programmatically creates a mailman mailist list and
#                  changes the privacy and general options of the list to match what
#                  is needed by the system.  It also adds the aliases to the sendmail
#                  aliases file.
#
# Parameters:
#                  listName:    The name of the list to create
#                  adminEmail:  The email of the list administrator
#                  password:    The administrative password for this list
#
#############################################################################################
    security.declareProtected('Admin KZMailman', 'createMailList')
    def createMailList(self, listName, adminEmail, password) :
       ''' Function creates maillist, aliases for mailing list and
           adds them to the aliases file.  The second task it serves is to
           post information to the maillist cgi-bin modules '''


       ###################################
       # call newlist command
       nlCommand = "%s/bin/newlist -q %s %s %s" % (self.mailmanPath, listName, adminEmail, password)

       ouptut, errors = self._runCommand(nlCommand)

       # get all newlist return codes
       if len(errors) > 0 : # success
         raise KMMException, errors
       else :
         ####################################
         # create aliases 
         if self.addAliases == 1 :
           self.createAliases(listName)
         return 0

#############################################################################################
# 
# createAliases : This function creates the necessary aliases in the mail programs aliases
#                 file.
#
# Parameters:
#                   serverName:  The server on which mailman is running
#                   listName:    The name of the mailing list to alter
#                   adminEmail:  The email of the administrative user.
#                   password:    The password of the administrative user.
#
#############################################################################################
    security.declarePrivate('createAliases')
    def createAliases(self, listName) :
       fAliases = open(self.aliasesPath, 'a')
       newLines = []
       mainListName = listName + "-list"
       adminListName = listName + "-admin"
       requestListName = listName + "-request"
       ownerListName = listName + "-owner"

       newLines.append('#################################################\n')
       lineText = '#Aliases for the %s list\n' % listName
       newLines.append(lineText)
    
       lineText = '%s:                   "|%s/mail/wrapper post %s"\n' % (mainListName, self.mailmanPath, listName)
       newLines.append(lineText)

       lineText = '%s:                   "|%s/mail/wrapper mailowner %s"\n' % (adminListName, self.mailmanPath, listName)
       newLines.append(lineText)
        
       lineText = '%s:                   "|%s/mail/wrapper mailcmd %s"\n' % (requestListName, self.mailmanPath, listName)
       newLines.append(lineText)
     
       lineText = '%s:                   %s\n' % (ownerListName, adminListName)
       newLines.append(lineText)
    
       fAliases.writelines(newLines)
       fAliases.close()



#############################################################################################
# 
# deleteMailList  : This function deletes a given Mailman mailing list.
#                  
#                  
#
# Parameters:
#             listName:         The name of the mailman to which a user needs to be added.
#             removeArchives:   A flag indicating whether or not the email list archives
#                               should also be deleted.
#
#############################################################################################
    security.declareProtected('Admin KZMailman', 'deleteMailList')
    def deleteMailList(self, listName, removeArchives=0) :
       ###################################
       # call rmlist command
       if removeArchives == None or removeArchives == 0 :
          nlCommand = "%s/bin/rmlist  %s" % (self.mailmanPath, listName)
       else :
          nlCommand = "%s/bin/rmlist -a %s" % (self.mailmanPath, listName)

       output, errors = self._runCommand(nlCommand)

       # get all newlist return codes
       if len(errors) > 0 : # error
         raise KMMException, errors
       else :
         return 0



#############################################################################################
# 
# addUserToList  : This function adds a user to a given Mailman mailing list.
#                  
#                  
#
# Parameters:
#             serverName:  The name of the server on which mailman is running.
#             listName:    The name of the mailman to which a user needs to be added.
#             password:    The administrative password for the list.
#             userEmail:   The email address that is to be added to the list.
#
#############################################################################################
    security.declareProtected('Add Mailman List Member', 'addMember')
    def addMember(self, listName, password, userEmail, sendWelcome) :
      ''' Function creates maillist, aliases for mailing list and
          adds them to the aliases file.  It then calls
          newaliases.  The second task it serves is to
          post information to the maillist cgi-bin modules '''

      
      tmpFile = tempfile.mktemp()
      inFile = open(tmpFile, 'w')
      inFile.write(userEmail)
      inFile.close()

      if sendWelcome == None or sendWelcome == 0 :
        nlCommand = "%s/bin/add_members -n %s -w n %s" % (self.mailmanPath, tmpFile, listName)
      else :
        nlCommand = "%s/bin/add_members -n %s -w y %s" % (self.mailmanPath, tmpFile, listName)
        
      output, errors = self._runCommand(nlCommand)
      os.remove(tmpFile)
      return output, errors


    security.declareProtected('Remove Mailman List Member', 'removeMember')
    def removeMember(self, listName, userEmail) :
       nlCommand = "%s/bin/remove_members %s %s" % (self.mailmanPath, listName, userEmail)
       output, errors = self._runCommand(nlCommand)
       return 0

    security.declareProtected('Get Mailman List Members', 'getMembers')
    def getMembers(self, listName, regular=0, digest=0) :
       ###################################
       # call rmlist command
       if regular != None and regular != 0 :
         rFlag = '-r'
       else :
         rFlag = ''
       if digest != None and digest != 0 :
         dFlag = '-d'
       else :
         dFlag = ''
       nlCommand = "%s/bin/list_members %s %s %s" % (self.mailmanPath, rFlag, dFlag, listName)

       output, errors = self._runCommand(nlCommand)

       # get all newlist return codes
       if len(errors) != 0 : # error
         raise KMMException, errors

       if len(output) > 0 :
         memberList = output.split('\n')
       else :
         memberList = []

       return memberList[0:-1]  # this strips off the last element which is always a blank string

    security.declareProtected('Admin KZMailman', 'getLists')
    def getLists(self) :
       ###################################
       # call list_lists command
       nlCommand = "%s/bin/list_lists" % (self.mailmanPath)

       output, errors = self._runCommand(nlCommand)

       # get all newlist return codes
       if len(errors) != 0 : # errors
         raise KMMException, errors

       newList = []
       if len(output) > 0 :
         listsList = output.split('\n')
         if len(listsList) > 0 :
           listsList = listsList[1:-1]
           for l in listsList :
             l = l.lstrip()
             listName = l[:l.find(' ')]
             newList.append(listName)

       return newList  # this strips off the last element which is always a blank string


    security.declarePrivate('deleteEmails')
    def deleteEmails(self, REQUEST=None) :
       keys = REQUEST.keys()
       delKeys = [key for key in keys if key[0:7] == 'delete_']
       userEmails = [key.split('_')[1] for key in delKeys]
       listName = REQUEST.get('listName')

       for user in userEmails :
          self.removeMember(listName, user)
       return

    security.declareProtected('Admin KZMailman', 'getListOptions')
    def getListOptions(self, listName) :
       # define omitted keys
       omittedKeys = ('bounce_matching_headers', 'msg_footer', 'digest_footer')
       # generate temp file name
       tmpFile = tempfile.mktemp()

       ###################################
       # call config_lists command
       nlCommand = "%s/bin/config_list -o %s %s" % (self.mailmanPath, tmpFile, listName)
       output, errors = self._runCommand(nlCommand)

       # get all newlist return codes
       if len(errors) > 0 : # error
         raise KMMException, errors

       options = {}
       file = open(tmpFile, 'r')
       for line in file.readlines() :
          if line[0] != '#' and len(line) > 1 and line.find('=') != -1:
            parts = line.split('=')
            if len(parts) > 1 :
              if parts[0].strip() not in omittedKeys :
                 try :
                   options[parts[0].strip()] = eval(parts[1].strip())
                 except :
                   pass
            else :
              options[parts[0].strip()] = ''
       
       file.close()
       os.remove(tmpFile)

       return options # this strips off the last element which is always a blank string

    security.declareProtected('Admin KZMailman', 'setListOptions')
    def setListOptions(self, listName, REQUEST=None) :
       """This method sets the options"""
       # build temp file
       tmpFile = tempfile.mktemp()
       file = open(tmpFile, 'w')

       for item in self.optionItems.keys() :
         if self.optionItems[item][2] == 'string' :
           line = "%s = '%s'\n" % (item, REQUEST.get(item))
         elif self.optionItems[item][2] == 'list' :
           value = REQUEST.get(item)
           if value == None or len(value) == 0 : value = '[]'
           line = "%s = %s\n" % (item, value)
         else :
           line = "%s = %s\n" % (item, REQUEST.get(item))

         file.write(line)

       file.close()

       ###################################
       # call config_lists command
       nlCommand = "%s/bin/config_list -i %s %s" % (self.mailmanPath, tmpFile, listName)

       output, errors = self._runCommand(nlcommand)

       os.remove(tmpFile)

       # get all newlist return codes
       if retcode != 0 : # success
         raise KMMException, errors
    
       return self.manage_list_options(self, REQUEST, manage_tabs_message="List Properties updated.")  

    security.declarePrivate('_runCommand')
    def _runCommand(self, nlcommand) :
       outf, inf, errf = popen2.popen3(nlcommand)
       output = outf.read()
       errors = errf.read()
       outf.close()
       inf.close()
       errf.close()
       return output, errors


InitializeClass(KZMailman)
